(* ::Package:: *)

(***************************************************************************************
Package for quantum mechanics calculations for a single spin-1/2 system

Place this file in a directory "NMR" in the Applications directory in $BaseDirectory 
(/Library/Mathematica on Mac OS X) or $UserBaseDirectory (~/Library/Mathematica 
on Mac OS X).

To use, type:

<<NMR`oneSpinLib`

For names of symbols and functions exported for user, type:

Names["NMR`oneSpinLib`*"]

For information about symbols and functions, type, for instance:

?mean

By default, rotation matrices used in this package lead to "left hand" rotations:
Starting with magnetization along positive z-axis, a positive pi/2 pulse along
x'-axis moves magnetization to positive y'-axis.  A positive pi/2 pulse
along y'-axis moves magnetization to negative x'-axis. This corresponds to a positive
gyromagnetic ratio in a right-handed coordinate system.

Rotations can be made to follow a right-hand convention by setting the variable lhr to -1.
Reset lhr to 1 to return to left-hand rotations.

Functions are provided to manipulate either a wavefunction or a density matrix.  
Wavefunctions and their functions are generally identified by "psi" and 
density matrices by "rho"

David P. Goldenberg
29 November 2009

**********************************************************************************)
BeginPackage["NMR`oneSpinLib`"]

(* This causes matrices to be automatically output in a nice format *)
$PrePrint = If[MatrixQ[#], MatrixForm[#], #] &

(* Begin list of exported functions with usage information *)

(* Density Matrix *)

rho::usage="rho[psi] returns density matrix corresponding to a pure population with wavefunction psi.  Psi is assumed to be a vector containing\!\(\*
StyleBox[\" \", \"Program\"]\)the complex coefficients of the basis set |a> and |b>"

(* Functions to calculate mean magnetization 
components from wavefunction or density matrix*)

meanPsi::usage="meanPsi[op,psi] calculates mean value predicted for the measurement associated with the operator, op, when applied to the wavefunction psi experessed as a vector containing complex coefficients of the basis set
				Caution: All variables defining the wavefunction are assumed to be real valued, except for the special variables ca and cb used for the general form of a wavefunction. "

meanRho::usage="meanRho[op,rho] calculates mean value predicted for the measurement associated with the operator, op, when applied to the density  matrix, rho.
				Caution: All variables defining the density matrix are assumed to be real valued, except for the special variables ca and cb used for the general form of a wavefunction. "

allMagPsi::usage ="allMagPsi[psi] writes output with all three magnetization components when applied to a wavefunction psi"

allMagRho::usage ="allMagRho[rho] writes output with all three magnetization components when applied to a density matrix rho"			


(* Functions to calculate change in wavefunction due to pulses *)

psiPulseX::usage="psiPulseX[psi, a] calculates the wavefunction generated by applying a pulse of angle a along the x axis to population defined by initial wavefunction psi"

psiPulseY::usage="psiPulseY[psi, a] calculates the wavefunction generated by applying a pulse of angle a along the y axis to populationdefined by initial wavefunction psi"
					
psiPi2X::usage="psiPi2X[psi] calculates the wavefunction generated by applying a Pi/2 pulse along the x axis to populationdefined by initial wavefunction psi"
					
psiPiX::usage="psiPiX[psi]calculates the wavefunction generated by applying a Pi pulse along the x axis to population defined by initial wavefunction psi"					

psiPi2Y::usage="psiPi2Y[psi] calculates the wavefunction generated by applying a Pi/2 pulse along the y axis to population defined by initial wavefunction psi"
					
psiPiY::usage="psiPiY[psi] calculates the wavefunction generated by applying a Pi pulse along the y axis to population defined by initial wavefunction psi"
					
(* Functions to calculate change in density matrix due to pulses *)		

rhoPulseX::usage="rhoPulseX[rho,a] calculates the density matrix generated by applying a pulse of angle a along the x axis to population defined by initial density matrix rho"

rhoPulseY::usage="rhoPulseY[rho,a] calculates the density matrix generated by applying a pulse of angle a along the y axis to population defined by initial density matrix rho"

rhoPi2X::usage="rhoPi2X[rho] calculates the density matrix generated by applying a Pi/2 pulse along the x axis to population defined by initial density matrix rho"

rhoPiX::usage="rhoPiX[rho] calculates the density matrix generated by applying a Pi pulse along the x axis to population defined by initial density matrix rho"
					
rhoPi2Y::usage="rhoPi2Y[rho]  calculates the density matrix generated by applying a Pi/2 pulse along the y axis to population defined by initial density matrix rho"

rhoPiY::usage="rhoPiY[rho]  calculates the density matrix generated by applying a Pi pulse along the y axis to population defined by initial density matrix rho"
			

(* Function to calculate change in wavefunction due to time evolution *)

psiTime::usage="psiTime[t, psi] calculates the wavefunction generated by an evolution period of t, starting with the initial wavefunction psi"


(* Function to calculate change in density matrix due to time evolution *)

rhoTime::usage="rhoTime[psi,t] calculates the density matrix generated by an evolution period of t, starting with the initial density matrix rho"


(* Equilibrium density matrix *)
rhoEq::usage = "rhoEq gives density matrix at thermal equilibrium"


psiGen::usage = "General form of wavefunction in terms of complex coefficients ca and cb"
psiA::usage = "Vector form of basis wavefunction |a>"
psiB::usage = "Vector form of basis wavefunction |b>"


(*Definitions of matrices and functions *)



(* Some wavefunctions *)
psiGen = {ca, cb}
psiA = {1, 0}
psiB = {0, 1}

(* Magnetization Operator Matrices *)

Ix = (1/2) {{0, 1}, {1, 0}} 
Iy = (I/2){{0, -1}, {1, 0}} 
Iz = (1/2) {{1, 0}, {0, -1}} 

(* Rotation Matrices *)

(* A constant to switch from left-hand to right-hand convention for pulses and
chemical-shift evolution Set lhr = -1 for right-hand rule *)

lhr = 1

Rx[a_] := {{Cos[a/2], lhr I Sin[a/2]}, {lhr I Sin[a/2], Cos[a/2]}}
Rinvx[a_] := {{Cos[a/2], (-lhr I)*Sin[a/2]}, {(-lhr I)*Sin[a/2], Cos[a/2]}}

Ry[a_] :=  {{Cos[a/2], lhr Sin[a/2]}, {-lhr Sin[a/2], Cos[a/2]}}
Rinvy[a_] := {{Cos[a/2], -lhr Sin[a/2]}, {lhr Sin[a/2], Cos[a/2]}}

(* Hamiltonian Matrix *)
H[t_, nu_] := {{Exp[lhr I Pi nu t], 0}, {0, Exp[-lhr I Pi nu t]}}
Hinv[t_, nu_] :={{Exp[-lhr I Pi nu t], 0}, {0, Exp[lhr I Pi nu t]}}

(* Functions for wavefunctions *)
meanPsi[op_, psi_] := ComplexExpand[Conjugate[psi].op.psi,{ca,cb},TargetFunctions->Conjugate]

allMagPsi[psi_] := Module [{},
    Print["<Ix> = " ,meanPsi[Ix,psi]];
    Print["<Iy> = " , meanPsi[Iy,psi]];
    Print["<Iz> = " , meanPsi[Iz,psi]];
    ]

psiPulseX[psi_,a_] := Rx[a].psi
psiPulseY[psi_,a_] := Ry[a].psi
psiPi2X[psi_] := psiPulseX[psi, Pi/2]
psiPiX[psi_] := psiPulseX[psi, Pi]
psiPi2Y[psi_] := psiPulseY[psi, Pi/2]
psiPiY[psi_] := psiPulseY[psi, Pi]

psiTime[psi_,t_] := H[t, nu].psi

(* Construct a density matrix from a wavefunction *)
rho[psi_] := Outer[Times, psi, Conjugate[psi]]

(* Equilibrium and general density matrices *)
rhoEq = deltaP Iz

(* Functions for density matrix *)
meanRho[op_, rho_] := Simplify[ComplexExpand[Tr[op.rho],{ca,cb},TargetFunctions->Conjugate]]

allMagRho[rho_] := Module [{},
    Print["<Ix> = " ,meanRho[Ix,rho]];
    Print["<Iy> = " , meanRho[Iy,rho]];
    Print["<Iz> = " , meanRho[Iz,rho]];
    ]

rhoPulseX[rho_,a_] := Rx[a].rho.Rinvx[a]
rhoPulseY[rho_, a_] := Ry[a].rho.Rinvy[a]
rhoPi2X[rho_] := rhoPulseX[rho, Pi/2]
rhoPiX[rho_] := rhoPulseX[rho, Pi]
rhoPi2Y[rho_] := rhoPulseY[rho, Pi/2]
rhoPiY[rho_] := rhoPulseY[rho, Pi]

rhoTime[rho_, t_] := H[t, nu].rho.Hinv[t, nu]



EndPackage[]



