(* ::Package:: *)

(***************************************************************************************
Package for quantum mechanics calculations of a weakly coupled 2-spin system.

Place this file in a directory "NMR" in the Applications directory in $BaseDirectory 
(/Library/Mathematica on Mac OS X) or $UserBaseDirectory (~/Library/Mathematica 
on Mac OS X).

To use, type:

<<NMR`twoSpinLib`

For names of symbols and functions exported for user, type:

Names["NMR`twoSpinLib`*"]

For information about symbols and functions, type, for instance:

?mean

By default, rotation matrices used in this package lead to "left hand" rotations:
Starting with magnetization along positive z-axis, a positive pi/2 pulse along
x'-axis moves magnetization to positive y'-axis.  A positive pi/2 pulse
along y'-axis moves magnetization to negative x'-axis. This corresponds to a positive
gyromagnetic ratio in a right-handed coordinate system.

Rotations can be made to follow a right-hand convention by setting the variable lhr to -1.
Reset lhr to 1 to return to left-hand rotations.

Functions are provided to manipulate either a wavefunction or a density matrix.  
Wavefunctions and their functions are generally identified by "psi" and 
density matrices by "rho"

David P. Goldenberg
29 November 2009
**************************************************************************************)


BeginPackage["NMR`twoSpinLib`"]

(* This causes matrices to be automatically output in a nice format *)
$PrePrint = If[MatrixQ[#],MatrixForm[#],#]&

(* Begin list of exported functions with usage information *)

(* Density Matrix *)

rho::usage="rho[psi] writes density matrix corresponding to a pure population with wave function psi, with psi expressed as a vector representing complex coefficients of basis set |aa>, |ab>, |ba> and |bb>"


(* Equilibrium density matrix *)
rhoEq::usage = "rhoEq gives density matrix at thermal equilibrium"


(* Functions to calculate mean magnetization components and product operators for I and S spin *)

meanPsi::usage="mean[op,psi] calculates mean value predicted for measurement associated with the operator, op, when applied to the wavefunction psi, with psi expressed as a vector representing complex coefficients of basis set"
				
meanRho::usage="meanRho[op,rho] calculates mean value predicted for the measurement associated with the operator, op, when applied to the density matrix, rho."

allMagPsi::usage ="allMagPsi[psi] writes output with all six magnetization components when applied to a wavefunction psi"

allMagRho::usage ="allMagRho[rho] writes output with all six magnetization components when applied to a density matrix rho"


opBasisRep::usage="opBasisRep[rho] writes output showing representation of density matrix rho as linear combination of operator matrices"



allCorPsi::usage="allCorPsi[psi] writes output with mean values of all 9 of the possible product operators applied to wavefunction psi"
				
allCorRho::usage="allCorRho[rho] writes output with mean values of all 9 of the possible product operators applied to density matrix rho"

(* Functions to calculate wavefunction following various pulses *)

(* Pulses with general duration defined by angle a *)

psiPulseXI::usage ="psiPulseXI[psi,a] calculates new wavefunction when pulse of angle a is applied along x'-axis to I spin of wavefunction psi"

psiPulseYI::usage ="psiPulseYI[psi,a] calculates new wavefunction when pulse of angle a is applied along y'-axis to I spin of wavefunction psi"

psiPulseXS::usage ="psiPulseXS[psi,a] calculates new wavefunction when pulse of angle a is applied along x'-axis to S spin of wavefunction psi"

psiPulseYS::usage ="psiPulseYS[psi,a] calculates new wavefunction when pulse of angle a is applied along y'-axis to S spin of wavefunction psi"

psiPulseX::usage ="psiPulseX[psi,a] calculates new wavefunction when nonselective pulse of angle a is applied along x'-axis to wavefunction psi"

psiPulseY::usage ="psiPulseY[psi,a] calculates new wavefunction when nonselective pulse of angle a is applied along y'-axis to wavefunction psi"

(* Pi/2 and Pi pulses *)
psiPi2XI::usage ="psiPi2XI[psi] calculates new wavefunction when pi/2 pulse is applied along x'-axis to I spin of wavefunction psi"

psiPi2YI::usage ="psiPi2YI[psi] calculates new wavefunction when pi/2 pulse is applied along y'-axis to I spin of wavefunction psi"

psiPiXI::usage ="psiPiXI[psi] calculates new wavefunction when pi pulse is applied along x'-axis to I spin of wavefunction psi"

psiPiYI::usage ="psiPiYI[psi] calculates new wavefunction when pi pulse is applied along y'-axis to I spin of wavefunction psi"

psiPi2XS::usage ="psiPi2XS[psi] calculates new wavefunction when pi/2 pulse is applied along x'-axis to S spin of wavefunction psi"

psiPi2YS::usage ="psiPi2YS[psi] calculates new wavefunction when pi/2 pulse is applied along y'-axis to S spin of wavefunction psi"

psiPiXS::usage ="psiPiXS[psi] calculates new wavefunction when pi pulse is applied along x'-axis to S spin of wavefunction psi"

psiPiYS::usage ="psiPiYS[psi] calculates new wavefunction when pi pulse is applied along y'-axis to S spin of wavefunction psi"

psiPi2X::usage ="psiPi2X[psi] calculates new wavefunction when non-selective pi/2 pulse is applied along x'-axis to wavefunction psi"

psiPi2Y::usage ="psiPi2Y[psi] calculates new wavefunction when non-selective pi/2 pulse is applied along y'-axis to wavefunction psi"

psiPiX::usage ="psiPiX[psi] calculates new wavefunction when non-selective pi pulse is applied along x'-axis to wavefunction psi"

psiPiY::usage ="psiPiY[psi] calculates new wavefunction when non-selective pi pulse is applied along y'-axis to wavefunction psi"


(* Functions to calculate change in density matrix due to pulses *)		

rhoPulseXI::usage ="rhoPulseXI[rho,a] calculates new density matrix when pulse of angle a is applied along x'-axis to I spin of density matrix rho"

rhoPulseYI::usage ="rhoPulseYI[rho,a] calculates new density matrix when pulse of angle a is applied along y'-axis to I spin of density matrix rho"

rhoPulseXS::usage ="rhoPulseXS[rho,a] calculates new density matrix when pulse of angle a is applied along x'-axis to S spin of density matrix rho"

rhoPulseYS::usage ="rhoPulseYS[rho,a] calculates new density matrix when pulse of angle a is applied along y'-axis to S spin of density matrix rho"

rhoPulseX::usage ="rhoPulseX[rho,a] calculates new density matrix when nonselective pulse of angle a is applied along x'-axis to density matrix rho"

rhoPulseY::usage ="rhoPulseY[rho,a] calculates new density matrix when nonselective pulse of angle a is applied along y'-axis to density matrix rho"

(* Pi/2 and Pi pulses *)
rhoPi2XI::usage ="rhoPi2XI[rho] calculates new density matrix when pi/2 pulse is applied along x'-axis to I spin of density matrix rho"

rhoPi2YI::usage ="rhoPi2YI[rho] calculates new density matrix when pi/2 pulse is applied along y'-axis to I spin of density matrix rho"

rhoPiXI::usage ="rhoPiXI[rho] calculates new density matrix when pi pulse is applied along x'-axis to I spin of density matrix rho"

rhoPiYI::usage ="rhoPiYI[rho] calculates new density matrix when pi pulse is applied along y'-axis to I spin of density matrix rho"

rhoPi2XS::usage ="rhoPi2XS[rho] calculates new density matrix when pi/2 pulse is applied along x'-axis to S spin of density matrix rho"

rhoPi2YS::usage ="rhoPi2YS[rho] calculates new density matrix when pi/2 pulse is applied along y'-axis to S spin of density matrix rho"

rhoPiXS::usage ="rhoPiXS[rho] calculates new density matrix when pi pulse is applied along x'-axis to S spin of density matrix rho"

rhoPiYS::usage ="rhoPiYS[rho] calculates new density matrix when pi pulse is applied along y'-axis to S spin of density matrix rho"

rhoPi2X::usage ="rhoPi2X[rho] calculates new density matrix when non-selective pi/2 pulse is applied along x'-axis to density matrix rho"

rhoPi2Y::usage ="rhoPiY[rho] calculates new density matrix when non-selective pi/2 pulse is applied along y'-axis to density matrix rho"

rhoPiX::usage ="rhoPiX[rho] calculates new density matrix when non-selective pi pulse is applied along x'-axis to density matrix rho"

rhoPiY::usage ="rhoPiY[rho] calculates new density matrix when non-selective pi pulse is applied along y'-axis to density matrix rho"


(* Function to calculate wavefunction following free precession*)

psiTime::usage ="psiTime[psi,t] calculates new wavefunction following time evolution for time t in static magnetic field, starting with wavefunction psi"

(* Function to calculate change in density matrix due to time evolution *)

rhoTime::usage="rhoTime[rho,t] calculates the density matrix generated by an evolution period of t, starting with the initial density matrix rho" (* Functions to calculate separate effects of chemical shift evolution and scalar coupling*)

psiTimeC::usage ="psiTimeC[psi,t] calculates new wavefunction following chemical shift evolution for time t in static magnetic field, starting with wavefunction psi"

psiTimeS::usage ="psiTimeS[psi,t] calculates new wavefunction following scalar coupling evolution for time t in static magnetic field, starting with wavefunction psi"

rhoTimeC::usage="rhoTimeC[rho,t] calculates the density matrix generated by a chemical shift evolution period of t, starting with the initial density matrix rho"

rhoTimeS::usage="rhoTimeS[rho,t] calculates the density matrix generated by a scalar couplling evolution period of t, starting with the initial density matrix rho"


(* Vector representations of eigenstates *)
psiAA::usage = "Vector representation of eigensate |aa> = {1,0,0,0}"
psiAB::usage = "Vector representation of eigensate |ab> = {0,1,0,0}"
psiBA::usage = "Vector representation of eigensate |ba> = {0,0,1,0}"
psiBB::usage = "Vector representation of eigensate |bb> = {0,0,0,1}"




(* Density Matrix for population with a single wavefunction *)

rho[psi_] := Outer[Times,psi,Conjugate[psi]]

(* General form of wavefunction and density matrix *)
psiGen = {caa,cab,cba,cbb}

(* Equilibrium density matrix *)
(* deltaPi = deltaEi/(2kT) *)
(* deltaPs = deltaEs/(2kT) *)
rhoEq = (deltaPi/2) Iz + (deltaPs/2) Sz

(* Vector representations of eigenstates *)

psiAA = {1,0,0,0}
psiAB = {0,1,0,0}
psiBA = {0,0,1,0}
psiBB = {0,0,0,1}

(* Matrix Representation of Magnetization Operators *)

(* Ix *)
Ix=(1/2)*({
 {0, 0, 1, 0},
 {0, 0, 0, 1},
 {1, 0, 0, 0},
 {0, 1, 0, 0}
});
              
   
(* Iy *)   
Iy=(1/2)*({
 {0, 0, -I, 0},
 {0, 0, 0, -I},
 {I, 0, 0, 0},
 {0, I, 0, 0}
});

(* Iz *)              
Iz=(1/2)*({
 {1, 0, 0, 0},
 {0, 1, 0, 0},
 {0, 0, -1, 0},
 {0, 0, 0, -1}
});

(* Sx *)              
Sx=(1/2)*({
 {0, 1, 0, 0},
 {1, 0, 0, 0},
 {0, 0, 0, 1},
 {0, 0, 1, 0}
});  

(* Sy *)              
Sy=(1/2)*({
 {0, -I, 0, 0},
 {I, 0, 0, 0},
 {0, 0, 0, -I},
 {0, 0, I, 0}
});

(* Sz *)              
Sz=(1/2)*({
 {1, 0, 0, 0},
 {0, -1, 0, 0},
 {0, 0, 1, 0},
 {0, 0, 0, -1}
});              
              

(* Product Operators *)

IxSx = Ix.Sx
IxSy = Ix.Sy
IxSz = Ix.Sz
IySx = Iy.Sx
IySy = Iy.Sy
IySz = Iy.Sz
IzSx = Iz.Sx
IzSy = Iz.Sy
IzSz = Iz.Sz
              
(* Functions for calculating mean magnetization components and product operators
	from wavefunction psi *)                                      
              
meanPsi[op_,psi_] := Simplify[ComplexExpand[Conjugate[psi].op.psi,{caa,cab,cba,cbb},TargetFunctions->Conjugate]]

allMagPsi[psi_] := Module [{},
    Print["<Ix> = " ,meanPsi[Ix,psi],"     ",
       "<Sx> = " ,meanPsi[Sx,psi]];
    Print["<Iy> = " , meanPsi[Iy,psi],"     ",
      "<Sy> = " , meanPsi[Sy,psi]];
    Print["<Iz> = " , meanPsi[Iz,psi],"     ",
      "<Sz> = " , meanPsi[Sz,psi]];
    ]
    
allCorPsi[psi_]:=Module[{},
    Print["<IxSx> = ",meanPsi[IxSx,psi],"     ","<IxSy> = ",meanPsi[IxSy,psi],"     ",
      "<IxSz> = ",meanPsi[IxSz,psi]];
    Print["<IySx> = ",meanPsi[IySx,psi],"     ","<IySy> = ",meanPsi[IySy,psi],"     ",
      "<IySz> = ",meanPsi[IySz,psi]];
    Print["<IzSx> = ",meanPsi[IzSx,psi],"     ","<IzSy> = ",meanPsi[IzSy,psi],"     ",
      "<IzSz> = ",meanPsi[IzSz,psi]];
    ]


(* Functions for calculating mean magnetization components and product operators
	from density matrix rho*)                                      
              
meanRho[op_,rho_] := Factor[TrigReduce[Simplify[ComplexExpand[Tr[op.rho],{caa,cab,cba,cbb},TargetFunctions->Conjugate]]]]

allMagRho[rho_] := Module [{},
    Print["<Ix> = " ,meanRho[Ix,rho],"     ",
       "<Sx> = " ,meanRho[Sx,rho]];
    Print["<Iy> = " , meanRho[Iy,rho],"     ",
      "<Sy> = " , meanRho[Sy,rho]];
    Print["<Iz> = " , meanRho[Iz,rho],"     ",
      "<Sz> = " , meanRho[Sz,rho]];
    ]
    
allCorRho[rho_]:=Module[{},
    Print["<IxSx> = ",meanRho[IxSx,rho],"     ","<IxSy> = ",meanRho[IxSy,rho],"     ",
      "<IxSz> = ",meanRho[IxSz,rho]];
    Print["<IySx> = ",meanRho[IySx,rho],"     ","<IySy> = ",meanRho[IySy,rho],"     ",
      "<IySz> = ",meanRho[IySz,rho]];
    Print["<IzSx> = ",meanRho[IzSx,rho],"     ","<IzSy> = ",meanRho[IzSy,rho],"     ",
      "<IzSz> = ",meanRho[IzSz,rho]];
    ]

opBasisRep[rho_]:=Module[{i,
	opNames={"Ix","Iy","Iz","Sx","Sy","Sz",
		"IxSx","IxSy","IxSz","IySx","IySy","IySz",
		"IzSx","IzSy","IzSz"},
	opValues={meanRho[Ix,rho],meanRho[Iy,rho],meanRho[Iz,rho], 
	meanRho[Sx,rho],meanRho[Sy,rho],meanRho[Sz,rho],
		4 meanRho[IxSx,rho],4 meanRho[IxSy,rho],4 meanRho[IxSz,rho],
		4 meanRho[IySx,rho],4 meanRho[IySy,rho],4meanRho[IySz,rho],
		4meanRho[IzSx,rho],4meanRho[IzSy,rho],4meanRho[IzSz,rho]}},
	For[i=1,i<16,i++,
		If[NumericQ[Part[opValues,i]]&& Part[opValues,i]== 0 ,Null,
			Print[Part[opValues,i],Part[opNames,i]]
		]
	]
]


(* Rotation operator matrices for calculating effects of pulses *)

(* Constant, lhr, to define rotation convention.  For left-hand rule rotations, lhr=1 *)
(* For right-hand rule, lhr = -1 *)

lhr=1



(* Rotation of I magnetizatio along x'-axis *)
RIx[a_] :={
	{Cos[a/2],0,lhr*I*Sin[a/2],0},
	{0,Cos[a/2],0,lhr*I*Sin[a/2]},
	{lhr*I*Sin[a/2],0,Cos[a/2],0},
	{0,lhr*I*Sin[a/2],0,Cos[a/2]}
	}
	
RIxInv[a_] := {{Cos[a/2], 0, (-I)*lhr*Sin[a/2], 0}, 
   {0, Cos[a/2], 0, (-I)*lhr*Sin[a/2]}, {(-I)*lhr*Sin[a/2], 0, Cos[a/2], 0}, 
   {0, (-I)*lhr*Sin[a/2], 0, Cos[a/2]}}
	
(* Rotation of I magnetizatio along y'-axis *)

RIy[a_] := {
	{Cos[a/2], 0, lhr*Sin[a/2], 0}, 
	{0, Cos[a/2], 0, lhr*Sin[a/2]}, 
	{-lhr*Sin[a/2], 0, Cos[a/2], 0}, 
	{0, -lhr*Sin[a/2], 0, Cos[a/2]}
	}

RIyInv[a_] := {{Cos[a/2], 0, -lhr*Sin[a/2], 0}, 
	{0, Cos[a/2], 0, -lhr*Sin[a/2]}, 
	{lhr*Sin[a/2], 0, Cos[a/2], 0}, 
	{0, lhr*Sin[a/2], 0, Cos[a/2]}}

(* Rotation of S magnetizatio along x'-axis *)

RSx[a_] := {
 	{Cos[a/2], I*lhr*Sin[a/2], 0, 0},
 	{I*lhr*Sin[a/2], Cos[a/2], 0, 0},
 	{0, 0, Cos[a/2], I*lhr*Sin[a/2]},
 	{0, 0, I*lhr*Sin[a/2], Cos[a/2]}}

RSxInv[a_] := {{Cos[a/2], (-I)*lhr*Sin[a/2], 0, 0}, 
	{(-I)*lhr*Sin[a/2], Cos[a/2], 0, 0}, 
	{0, 0, Cos[a/2], (-I)*lhr*Sin[a/2]}, 
	{0, 0, (-I)*lhr*Sin[a/2], Cos[a/2]}}

(* Rotation of S magnetizatio along y'-axis *)            

RSy[a_] := {
	{Cos[a/2], lhr*Sin[a/2], 0, 0}, 
	{-lhr*Sin[a/2], Cos[a/2], 0, 0}, 
	{0, 0, Cos[a/2], lhr*Sin[a/2]}, 
   {0, 0, -lhr*Sin[a/2], Cos[a/2]}
   }


RSyInv[a_] := {{Cos[a/2], -lhr*Sin[a/2], 0, 0}, 
	{lhr*Sin[a/2], Cos[a/2], 0, 0}, 
	{0, 0, Cos[a/2], -lhr*Sin[a/2]}, 
	{0, 0, lhr*Sin[a/2], Cos[a/2]}}
        
(* Hamiltonian Matrix *)
H[t_, nuI_, nuS_,J_] := {{Exp[I lhr Pi (nuI+nuS-J/2)t], 0, 0, 0},
						 {0,Exp[I lhr Pi (nuI-nuS+J/2)t],0 ,0},         
						 {0,0, Exp[I lhr Pi (-nuI+nuS+J/2)t],0},
						 {0,0,0, Exp[I lhr Pi (-nuI-nuS-J/2)t]}}
						 
Hinv[t_, nuI_, nuS_,J_] := {{Exp[-I lhr Pi (nuI+nuS-J/2)t], 0, 0, 0},
						 {0,Exp[-I lhr Pi (nuI-nuS+J/2)t],0 ,0},         
						 {0,0, Exp[-I lhr Pi (-nuI+nuS+J/2)t],0},
						 {0,0,0, Exp[-I lhr Pi (-nuI-nuS-J/2)t]}}

(* Separation of the Hamiltonian into chemical shift a scalar coupling terms *)

Hc[t_,nuI_,nuS_] := H[t,nuI,nuS,J]/. J-> 0

HcInv[t_,nuI_,nuS_] := Hinv[t,nuI,nuS,J]/. J-> 0

Hs[t_,J_] := H[t,nuI,nuS,J] /.{ nuI-> 0,nuS-> 0}

HsInv[t_, J_] :=  Hinv[t,nuI,nuS,J]/.{ nuI-> 0,nuS-> 0}

(* Functions for calculating wavefunction following a pulse *)

(* General pulses along x'- or y'-axis of duration angle a *)
psiPulseXI[psi_,a_] := RIx[a] . psi ;
psiPulseYI[psi_,a_] := RIy[a] . psi ;
psiPulseXS[psi_,a_] := RSx[a] . psi ;
psiPulseYS[psi_,a_] := RSy[a] . psi ;

psiPulseX[psi_,a_] := psiPulseXI[psiPulseXS[psi,a],a]
psiPulseY[psi_,a_] := psiPulseYI[psiPulseYS[psi,a],a]


(* Pi or Pi/2 pulses along x'- or y'-axis *)

psiPi2XI[psi_] := RIx[Pi/2] . psi
psiPi2YI[psi_] := RIy[Pi/2]. psi
psiPiXI[psi_] := RIx[Pi] . psi
psiPiYI[psi_] := RIy[Pi]. psi
psiPi2XS[psi_] := RSx[Pi/2] . psi
psiPi2YS[psi_] := RSy[Pi/2] . psi
psiPiXS[psi_] := RSx[Pi] . psi
psiPiYS[psi_] := RSy[Pi] . psi

psiPi2X[psi_] := psiPi2XI[psiPi2XS[psi]]
psiPi2Y[psi_] := psiPi2YI[psiPi2YS[psi]]
psiPiX[psi_] := psiPiXI[psiPiXS[psi]]
psiPiY[psi_] := psiPiYI[psiPiYS[psi]]

(* Functions for calculating density matrix following a pulse *)

(* General pulses along x'- or y'-axis of duration angle a *)
rhoPulseXI[rho_,a_] := RIx[a] . rho . Inverse[RIx[a]];
rhoPulseYI[rho_,a_] := RIy[a] . rho . Inverse[RIy[a]];
rhoPulseXS[rho_,a_] := RSx[a] . rho . Inverse[RSx[a]];
rhoPulseYS[rho_,a_] := RSy[a] . rho . Inverse[RSy[a]];

rhoPulseX[rho_,a_] := rhoPulseXI[rhoPulseXS[rho,a],a]
rhoPulseY[rho_,a_] := rhoPulseYI[rhoPulseYS[rho,a],a]


(* Pi or Pi/2 pulses along x'- or y'-axis *)

rhoPi2XI[rho_] := rhoPulseXI[rho,Pi/2]
rhoPi2YI[rho_] := rhoPulseYI[rho,Pi/2]
rhoPiXI[rho_] := rhoPulseXI[rho,Pi]
rhoPiYI[rho_] := rhoPulseYI[rho,Pi]
rhoPi2XS[rho_] := rhoPulseXS[rho,Pi/2]
rhoPi2YS[rho_] :=rhoPulseYS[rho,Pi/2]
rhoPiXS[rho_] := rhoPulseXS[rho,Pi]
rhoPiYS[rho_] := rhoPulseYS[rho,Pi]

rhoPi2X[rho_] := rhoPulseX[rho,Pi/2]
rhoPi2Y[rho_] := rhoPulseY[rho,Pi/2]
rhoPiX[rho_] := rhoPulseX[rho,Pi]
rhoPiY[rho_] := rhoPulseY[rho,Pi]


(* Time Evolution *)

psiTime[psi_,t_] := H[t,nuI,nuS,J].psi
rhoTime[rho_,t_] := H[t,nuI,nuS,J].rho.Hinv[t,nuI,nuS,J]

(* Chemical shift evolution *)
psiTimeC[psi_,t_] := Hc[t,nuI,nuS,J].psi 
rhoTimeC[rho_,t_] := Hc[t,nuI,nuS].rho.HcInv[t,nuI,nuS]

(* Scalar coupling evolution *)
psiTimeS[psi_,t_] := Hs[t,nuI,nuS,J].psi 
rhoTimeS[rho_,t_] := Hs[t,J].rho.HsInv[t,J]

EndPackage[]



